#include <stdio.h>
#include <string.h>
#include "../sockev.h"
#include "../sockev_tcp4.h"
#include "../filt_split.h"
#include "../filt_sessions.h"

int lport = 4242;
int stage = 0;

sockev_t sa, sc;

sockev_filters_t flt_sess[] = {
	{sockev_filter_session, NULL},
	{NULL, NULL}
};

sockev_filters_t flt_linesplit[] = {
	{sockev_filter_split, "\r\n"},
	{NULL, NULL}
};

/* accepted sockets */
void ev_acc_open(sockev_t id)
{
	printf("ACC [%d] open\n", id);
	stage++;
}

void ev_acc_read(sockev_t id, char *buff, int len)
{
	printf("ACC [%d] read: '%s'/%d\n", id, buff, len);
}

void ev_acc_close(sockev_t id)
{
	printf("ACC [%d] close\n", id);
}

sockev_hooks_t ev_acc = {ev_acc_open, ev_acc_close, NULL, NULL, NULL, ev_acc_read, NULL};

/* listen socket */
void ev_lst_open(sockev_t id)
{
	printf("LST [%d] open, listening\n", id);
}

int ev_lst_readable(sockev_t id)
{
	printf("LST [%d] incoming\n", id);
	sa = sockev_tcp4_accept(sockev_new(&ev_acc, 0, flt_sess), id);
	printf("LST [%d] new=%d\n", id, sa);
	if (sa < 0)
		printf("UNABLE TO ACCEPT\n");
	return 1; /* event handled, no further action is required from libsockev */
}

void ev_lst_close(sockev_t id)
{
	printf("LST [%d] close\n", id);
}

sockev_hooks_t ev_lst = {ev_lst_open, ev_lst_close, NULL, ev_lst_readable, NULL, NULL, NULL};


/* connect socket */
void ev_con_open(sockev_t id)
{
	printf("CON [%d] open\n", id);
	stage++;
}

void ev_con_close(sockev_t id)
{
	printf("CON [%d] close\n", id);
}

void ev_con_read(sockev_t id, char *buff, int len)
{
	printf("CON [%d] read: '%s'/%d\n", id, buff, len);
}

sockev_hooks_t ev_con = {ev_con_open, ev_con_close, NULL, NULL, NULL, ev_con_read, NULL};

/* session socket */
void ev_sess_open(sockev_t id)
{
	printf("SESS [%d %s] open\n", id, (char *)sockev_get_userdata(id));
	stage++;
}

void ev_sess_close(sockev_t id)
{
	printf("SESS [%d %s] close\n", id, (char *)sockev_get_userdata(id));
}

void ev_sess_read(sockev_t id, char *buff, int len)
{
	printf("SESS [%d %s] read: '%s'/%d\n", id, (char *)sockev_get_userdata(id), buff, len);
}

sockev_hooks_t ev_sess = {ev_sess_open, ev_sess_close, NULL, NULL, NULL, ev_sess_read, NULL};


int main()
{
	int n;

	sockev_init();
	if (sockev_tcp4_listen(sockev_new(&ev_lst, 0, NULL), "127.0.0.1", lport, 10) == SOCKEV_INVALID) {
		fprintf(stderr, "test_net: can't open listening socket on port %d\n", lport);
		return 1;
	}

	if ((sc = sockev_tcp4_connect(sockev_new(&ev_con, 0, flt_sess), "127.0.0.1", lport)) == SOCKEV_INVALID) {
		fprintf(stderr, "test_net: can't connect socket on port %d\n", lport);
		return 1;
	}
	printf("main loop begin\n");
	for(n = 0; n < 8; n++) {
		printf("%d\n", stage);
		if (stage == 2) {
			sockev_t s;
			char name[1024];
			
			s = sockev_new(&ev_sess, 0, flt_linesplit);
			sprintf(name, "{%d:%d}", sa, 42);
			sockev_set_userdata(s, strdup(name));
			sockev_filter_session_attach(sa, 42, s);
			printf("attach: %d\n", sa);
			stage++;
		}
		sockev_poll(1000);
	}
	printf("main loop end\n");
	sockev_uninit();
	return 0;
}
